@@ -6,7 +6,27 @@ from codebug_tether.i2c import *
 from codebug_tether.serial_channel_device import SerialChannelDevice
 
 
-DEFAULT_SERIAL_PORT = '/dev/ttyACM0'
+# setup default serial port (different on MacOS and >=Raspberry Pi 3)
+if os.uname().sysname == 'Darwin':
+    # OSX
+    DEFAULT_SERIAL_PORT = '/dev/tty.USBmodem'
+else:
+    # assume we're on Raspberry Pi/Linux
+    def get_rpi_revision():
+        """Returns the version number from the revision line."""
+        for line in open("/proc/cpuinfo"):
+            if "Revision" in line:
+                import re
+                return re.sub('Revision\t: ([a-z0-9]+)\n', r'\1', line)
+
+    rpi_revision = get_rpi_revision()
+    if rpi_revision and rpi_revision != 'Beta' and int('0x'+rpi_revision, 16) >= 0xa02082:
+        # RPi 3 and above
+        DEFAULT_SERIAL_PORT = '/dev/ttyS0'
+    else:
+        # RPi 2 and below
+        DEFAULT_SERIAL_PORT = '/dev/ttyACM0'
+
 
 IO_DIGITAL_OUTPUT = 0
 IO_DIGITAL_INPUT = 1
@@ -42,7 +62,9 @@ IO_PWM_OUTPUT = 3
  CHANNEL_INDEX_COLOURTAIL_CONTROL,
  CHANNEL_INDEX_PWM_CONF_0,
  CHANNEL_INDEX_PWM_CONF_1,
- CHANNEL_INDEX_PWM_CONF_2) = range(5, 30)
+ CHANNEL_INDEX_PWM_CONF_2,
+ CHANNEL_INDEX_SERVO_PULSE_LENGTH,
+ CHANNEL_INDEX_SERVO_CONF) = range(5, 32)
 
 EXTENSION_CONF_IO = 0x01
 EXTENSION_CONF_SPI = 0x02
@@ -170,7 +192,7 @@ class CodeBug(SerialChannelDevice):
         """Turns on the PWM generator with the given settings.
 
         :param t2_prescale: One of T2_PS_1_1, T2_PS_1_4, T2_PS_1_16
-                            Scales down the 4MHz instruction clock by
+                            Scales down the 12MHz instruction clock by
                             1, 4 or 16.
         :param full_period: 8-bit value - which is scaled up to 10-bits
                             (<< 2) - to which timer 2 will count up to
@@ -180,7 +202,7 @@ class CodeBug(SerialChannelDevice):
                           full_period to control duty cycle. For
                           example:
 
-                              # 4MHz / 16 with 50% duty cycle
+                              # 12MHz / 16 with 50% duty cycle
                               codebug.pwm_on(T2_PS_1_16, 0xff, 0x200)
 
         """
@@ -203,19 +225,32 @@ class CodeBug(SerialChannelDevice):
 
         """
         # calculate pwm settings
-        # 4 MHz / 16 = 250k ticks per second
-        full_period = int(250000 / frequency) - 1
+        # 12MHz / 16 = 750k ticks per second
+        full_period = int(750000 / frequency) - 1
         # for 50% duty cycle: shift up by 2 then /(2 i.e. 50% duty cycle)
         # on_period = (full_period << 2) / 2;
         # this is quicker
         on_period = full_period << 1
         self.pwm_on(T2_PS_1_16, full_period, on_period)
-        pass
 
     def pwm_off(self):
+        """Turns off the PWM generator."""
         go_busy_off_mask = 0xff ^ (1 << 4)
         self.and_mask(CHANNEL_INDEX_PWM_CONF_2, go_busy_off_mask)
 
+    def servo_set(self, servo_index, pulse_length):
+        """Set the servo at servo_index to pulse_length. Make sure that
+        the leg is configured as IO_DIGITAL_OUTPUT (0).
+        """
+        pulse_length_msb = 0xff & (pulse_length >> 8)
+        pulse_length_lsb = 0xff & pulse_length
+        conf_msb = ((servo_index & 0xf) << 4) | 0x01
+        conf_lsb = ((servo_index & 0xf) << 4) | 0x00
+        self.set_bulk(CHANNEL_INDEX_SERVO_PULSE_LENGTH,
+                      bytes([pulse_length_msb, conf_msb]))
+        self.set_bulk(CHANNEL_INDEX_SERVO_PULSE_LENGTH,
+                      bytes([pulse_length_lsb, conf_lsb]))
+
     def clear(self):
         """Clears the pixels on CodeBug.
 
@@ -543,3 +578,19 @@ class CodeBug(SerialChannelDevice):
     def uart_rx_get_buffer(self, length, offset=0):
         """Returns data bytes from UART buffer."""
         return self.get_buffer(UART_RX_BUFFER_INDEX, length, offset)
+
+
+def scale(x, from_low, from_high, to_low, to_high):
+    # Hardware can only do 16bit maths
+    def limit(v):
+        max_value = 0x7fff
+        min_value = -0x8000
+        return max(min_value, min(max_value, v))
+    x, from_low, from_high, to_low, to_high = map(
+        limit, (x, from_low, from_high, to_low, to_high))
+    # do the scale
+    from_delta = from_high - from_low
+    x_offset = x - from_low
+    to_delta = to_high - to_low
+    new_x = int((x_offset * to_delta) / from_delta)
+    return to_low + new_x
